@@ -5,7 +5,7 @@
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: rpcdump.py 706 2012-08-30 20:25:03Z bethus@gmail.com $
+# $Id: rpcdump.py 1076 2014-01-09 19:17:14Z bethus@gmail.com $
 #
 # DCE/RPC endpoint mapper dumper.
 #
@@ -22,7 +22,8 @@ import sys
 import types
 
 from impacket import uuid, ntlm, version
-from impacket.dcerpc import dcerpc_v4, dcerpc, transport, epm, ndrutils
+from impacket.dcerpc.v5 import transport, epm
+from impacket.dcerpc import  ndrutils
 import argparse
 
 class RPCDump:
@@ -30,8 +31,6 @@ class RPCDump:
         '139/SMB': (r'ncacn_np:%s[\pipe\epmapper]', 139),
         '445/SMB': (r'ncacn_np:%s[\pipe\epmapper]', 445),
         '135/TCP': (r'ncacn_ip_tcp:%s', 135),
-        '135/UDP': (r'ncadg_ip_udp:%s', 135),
-        '80/HTTP': (r'ncacn_http:%s', 80),
         }
 
 
@@ -86,8 +85,8 @@ class RPCDump:
         endpoints = {}
         # Let's groups the UUIDS
         for entry in entries:
-            binding = epm.PrintStringBinding(entry['Tower']['Floors'], rpctransport.get_dip())
-            tmpUUID = str(entry['Tower']['Floors'][0])
+            binding = epm.PrintStringBinding(entry['tower']['Floors'], rpctransport.get_dip())
+            tmpUUID = str(entry['tower']['Floors'][0])
             if endpoints.has_key(tmpUUID) is not True:
                 endpoints[tmpUUID] = {}
                 endpoints[tmpUUID]['Bindings'] = list()
@@ -95,7 +94,7 @@ class RPCDump:
                 endpoints[tmpUUID]['EXE'] = ndrutils.KNOWN_UUIDS[uuid.uuidtup_to_bin(uuid.string_to_uuidtup(tmpUUID))[:18]]
             else:
                 endpoints[tmpUUID]['EXE'] = 'N/A'
-            endpoints[tmpUUID]['Annotation'] = entry['Annotation'][:-1]
+            endpoints[tmpUUID]['annotation'] = entry['annotation'][:-1]
             endpoints[tmpUUID]['Bindings'].append(binding)
 
             if epm.KNOWN_PROTOCOLS.has_key(tmpUUID[:36]):
@@ -107,7 +106,7 @@ class RPCDump:
         for endpoint in endpoints.keys():
             print "Protocol: %s " % endpoints[endpoint]['Protocol']
             print "Provider: %s " % endpoints[endpoint]['EXE']
-            print "UUID    : %s %s" % (endpoint, endpoints[endpoint]['Annotation'])
+            print "UUID    : %s %s" % (endpoint, endpoints[endpoint]['annotation'])
             print "Bindings: "
             for binding in endpoints[endpoint]['Bindings']:
                 print "          %s" % binding
@@ -124,20 +123,15 @@ class RPCDump:
 
 
     def __fetchList(self, rpctransport):
-        # UDP only works over DCE/RPC version 4.
-        if isinstance(rpctransport, transport.UDPTransport):
-            dce = dcerpc_v4.DCERPC_v4(rpctransport)
-        else:
-            dce = dcerpc.DCERPC_v5(rpctransport)
-
+        dce = rpctransport.get_dce_rpc()
         entries = []
 
         dce.connect()
-        dce.set_auth_level(ntlm.NTLM_AUTH_PKT_PRIVACY)
-        dce.bind(epm.MSRPC_UUID_PORTMAP)
-        rpcepm = epm.DCERPCEpm(dce)
+        #dce.set_auth_level(ntlm.NTLM_AUTH_PKT_INTEGRITY)
+        #dce.bind(epm.MSRPC_UUID_PORTMAP)
+        #rpcepm = epm.DCERPCEpm(dce)
 
-        resp = rpcepm.lookup('', inquireType = epm.RPC_C_EP_ALL_ELTS)
+        resp = epm.hept_lookup(rpctransport.get_dip())
 
         dce.disconnect()
 
@@ -168,6 +162,10 @@ if __name__ == '__main__':
     if domain is None:
         domain = ''
 
+    if password == '' and username != '' and options.hashes is None:
+        from getpass import getpass
+        password = getpass("Password:")
+
     dumper = RPCDump(options.protocol, username, password, domain, options.hashes)
 
     dumper.dump(address)
