@@ -5,7 +5,7 @@
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: services.py 1075 2014-01-09 19:12:51Z bethus@gmail.com $
+# $Id: services.py 1203 2014-04-08 15:05:39Z bethus@gmail.com $
 #
 # [MS-SCMR] services common functions for manipulating services
 #
@@ -43,7 +43,7 @@ class SVCCTL:
         self.__lmhash = ''
         self.__nthash = ''
         if options.hashes is not None:
-            self.__lmhash, self.__nthash = hashes.split(':')
+            self.__lmhash, self.__nthash = options.hashes.split(':')
 
     def run(self, addr):
 
