@@ -4,7 +4,7 @@
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: ImpactPacket.py 946 2013-11-26 17:50:36Z aurelianocalvo $
+# $Id: ImpactPacket.py 1144 2014-02-17 20:27:11Z bethus@gmail.com $
 #
 # Description:
 #  Network packet codecs basic building blocks.
@@ -38,7 +38,7 @@ class ImpactPacketException(Exception):
     def __str__(self):
         return `self.value`
 
-class PacketBuffer:
+class PacketBuffer(object):
     """Implement the basic operations utilized to operate on a
     packet's raw buffer. All the packet classes derive from this one.
 
