@@ -4,7 +4,7 @@
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: ndrutils.py 644 2012-08-04 19:35:30Z bethus@gmail.com $
+# $Id: ndrutils.py 898 2013-11-05 23:38:21Z bethus@gmail.com $
 #
 
 from struct import *
@@ -592,6 +592,15 @@ class NDRStringA(Structure):
         ('Data',':'),
     )
 
+class NDRPointerNew(Structure):
+    commonHdr = (
+        ('RefId','<L=0xff'),
+    )
+    def __init__(self, data = None, alignment = 0):
+        Structure.__init__(self,data, alignment)
+        if data is None:
+            self['RefId'] = random.randint(1,65535)
+
 class NDRUniqueStringA(NDRStringA):
     commonHdr = (
         ('RefId','<L'),
@@ -621,3 +630,67 @@ class NDRUniqueStringW(NDRStringW):
         self['RefId'] = random.randint(1,65535)
 
 
+class pRPC_UNICODE_STRING(Structure):
+    # Pointer to a RPC_UNICODE_STRING
+    structure = (
+        ('Length','<H=0'),
+        ('MaximumLength','<H=0'),
+        ('NDRStringPtr','<L=0'),
+    )
+
+    def __init__(self, data = None, alignment = 0):
+        Structure.__init__(self, data, alignment)
+        self['NDRStringPtr'] = random.randint(1,65535)
+
+    def setDataLen(self, data):
+        self['Length'] = len(data)
+        self['MaximumLength'] = self['Length']
+
+class RPC_UNICODE_STRING(Structure):
+    # Same as NDRStringW except DataLen is taken from ActualCount
+    structure = (
+        ('MaxCount','<L=len(Data)/2'),
+        ('Offset','<L=0'),
+        ('ActualCount','<L=len(Data)/2'),
+        ('DataLen','_-Data','ActualCount*2'),
+        ('Data',':'),
+    )
+
+    def __init__(self, data = None, alignment = 0):
+        Structure.__init__(self, data, alignment)
+
+class NDRConformantVaryingArray(Structure):
+    structure = (
+        ('MaxCount','<L=len(Data)'),
+        ('Offset','<L=0'),
+        ('ActualCount','<L=len(Data)'),
+        ('DataLen','_-Data','ActualCount'),
+        ('Data',':'),
+    )
+
+# I'm not sure this will work as a general Array. For now I'm leaving this here
+class NDRArray(Structure):
+    structure = (
+        ('MaxCount','<L=0'),
+        ('Data', ':'),
+    )
+    def __init__(self, data = None, alignment = 0, itemClass = None):
+        if data is not None:
+            Structure.__init__(self,data,alignment)
+            for i in range(self['MaxCount']):
+                item = 'Item_%d'%i 
+                self[item] = itemClass(self['Data'])
+                self['Data'] = self['Data'][len(self[item]):]
+            # Now, we're past the structs headers, let's go to the deferred stuff
+            for i in range(self['MaxCount']):
+                item = 'Item_%d'%i 
+                prevLen = len(self[item])
+                self[item].fromStringDeferred(self['Data'])
+                self['Data'] = self['Data'][len(self[item])-prevLen:]
+
+    def __len__(self):
+        for i in range(self['MaxCount']):
+            item = 'Item_%d'%i 
+            finalLen = len(self[item])
+        return finalLen + 4
+
