@@ -4,7 +4,7 @@
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: svcctl.py 715 2012-09-10 19:40:25Z bethus@gmail.com $
+# $Id: svcctl.py 875 2013-10-29 14:33:35Z bethus@gmail.com $
 #
 # Author: Alberto Solino
 #
@@ -13,6 +13,7 @@
 #
 
 import array
+import random
 from struct import *
 from impacket import ImpactPacket
 from impacket.structure import Structure
@@ -344,6 +345,25 @@ class SVCCTLREnumServicesStatusWResponse(Structure):
         ('ErrorCode','<L'),
     )
 
+class SVCCTLRChangeServiceConfigW(Structure):
+     opnum = 11
+     alignment = 4
+     structure = (
+         ('ContextHandle','20s'),
+         ('ServiceType','<L=0xffffffff'),
+         ('StartType','<L=0xffffffff'),
+         ('ErrorControl','<L=0xffffffff'),
+         ('BinaryPathName',':'),
+         ('LoadOrderGroup','<L=0'),
+         ('TagID','<L=0'),
+         ('Dependencies','<L=0'),
+         ('DependenciesSize','<L=0'),
+         ('ServiceStartName',':'),
+         ('Password',':'),
+         ('PwSize','<L=0'),
+         ('DisplayName',':'),
+     )
+
 # OLD Style structs.. leaving this stuff for compatibility purpose. Don't use these structs/functions anymore
 
 class SVCCTLOpenSCManagerHeader(ImpactPacket.Header):
@@ -855,12 +875,27 @@ class DCERPCSvcCtl:
         return answer
 
     def DeleteService(self, handle):
+        """
+        deletes a service
+
+        :param HANDLE handle: a valid HANDLE to a service previously opened
+
+        :return: on error it raises an exception. Otherwise it was successful
+        """
+
         deleteService = SVCCTLRDeleteService()
         deleteService['ContextHandle'] = handle
         ans = self.doRequest(deleteService, checkReturn = 1)
         return ans
 
     def StopService(self, handle):
+        """
+        stops a service
+
+        :param HANDLE handle: a valid HANDLE to a service previously opened
+
+        :return: returns an updated SVCCTLServiceStatus structure. Call dump() method to see its contents. On error it raises an exception
+        """
         controlService = SVCCTLRControlService()
         controlService['ContextHandle'] = handle
         controlService['Control']  = SERVICE_CONTROL_STOP
@@ -868,6 +903,15 @@ class DCERPCSvcCtl:
         return SVCCTLServiceStatus(ans)
  
     def OpenServiceA(self, handle, name):
+        """
+        opens a service
+
+        :param HANDLE handle: a valid HANDLE to the SCM database (see OpenSCManagerW)
+        :param STRING name: name of the service
+
+        :return: returns an SVCCTLROpenServiceResponse structure with the service handle. Call dump() method to see its contents. On error it raises an exception
+
+        """
         openService = SVCCTLROpenServiceA()
         openService['SCManager'] = handle
         openService['ServiceName'] = ndrutils.NDRStringA()
@@ -878,6 +922,15 @@ class DCERPCSvcCtl:
         return SVCCTLROpenServiceResponse(ans)
 
     def OpenServiceW(self, handle, name):
+        """
+        opens a service
+
+        :param HANDLE handle: a valid HANDLE to the SCM database (see OpenSCManagerW)
+        :param UNICODE name: name of the service
+
+        :return: returns an SVCCTLROpenServiceResponse structure with the service handle. Call dump() method to see its contents. On error it raises an exception
+
+        """
         # We MUST receive Unicode data here
         openService = SVCCTLROpenServiceW()
         openService['SCManager'] = handle
@@ -889,6 +942,15 @@ class DCERPCSvcCtl:
         return SVCCTLROpenServiceResponse(ans)
 
     def StartServiceW(self, handle, arguments = ''):
+        """
+        starts a service
+
+        :param HANDLE handle: a valid HANDLE to the service (see OpenServiceW)
+        :param STRING arguments: arguments to be passed to the service
+
+        :return: On error it raises an exception. Otherwise it was successful
+        """
+
         # TODO: argv has to be a pointer to a buffer that contains an array
         # of pointers to null-terminated UNICODE strings that are passed as
         # arguments to the service
@@ -913,7 +975,18 @@ class DCERPCSvcCtl:
       
         return ans
 
-    def CreateServiceW(self, handle, serviceName, displayName, binaryPathName):
+    def CreateServiceW(self, handle, serviceName, displayName, binaryPathName, serviceType = SERVICE_WIN32_OWN_PROCESS):
+        """
+        creates a service
+
+        :param HANDLE handle: a valid HANDLE to the SCM database (see OpenSCManagerW)
+        :param UNICODE serviceName: the name of the service to create
+        :param UNICODE displayName: the display name of the service to create
+        :param UNICODE binaryPathName: the pathname for the binary to be executed when starting the service
+        :param INT serviceType: the type of service to be created. See service types within this file or [MS-SCMR] section 3.1.4.12.
+
+        :return: returns an SVCCTLRCreateServiceWResponse structure with the service handle. Call dump() method to see its contents. On error it raises an exception
+        """
         # We MUST receive Unicode data here
         createService = SVCCTLRCreateServiceW()
         createService['SCManager']      = handle
@@ -922,7 +995,7 @@ class DCERPCSvcCtl:
         createService['DisplayName']    = ndrutils.NDRUniqueStringW()
         createService['DisplayName']['Data']    = (displayName+'\x00'.encode('utf-16le'))
         createService['DesiredAccess']  = SERVICE_ALL_ACCESS
-        createService['ServiceType']    = SERVICE_WIN32_OWN_PROCESS | SERVICE_INTERACTIVE_PROCESS
+        createService['ServiceType']    = serviceType
         createService['StartType']      = SERVICE_AUTO_START
         #createService['StartType']      = SERVICE_DEMAND_START
         createService['ErrorControl']   = SERVICE_ERROR_IGNORE
@@ -933,6 +1006,11 @@ class DCERPCSvcCtl:
         return SVCCTLRCreateServiceWResponse(ans)
 
     def OpenSCManagerW(self): 
+        """
+        opens the SCM database on the specified server.
+
+        :return: returns an SVCCTLROpenSCManagerAResponse structure with the SCM handle. Call dump() method to see its contents. On error it raises an exception
+        """
         openSCManager = SVCCTLROpenSCManagerW()
         openSCManager['MachineName'] = ndrutils.NDRUniqueStringW()
         openSCManager['MachineName']['Data'] = 'DUMMY\x00'.encode('utf-16le')
@@ -942,12 +1020,38 @@ class DCERPCSvcCtl:
         return SVCCTLROpenSCManagerAResponse(ans)
 
     def CloseServiceHandle(self, handle):
+        """
+        releases the handle to the specified service or the SCM database
+
+        :param HANDLE handle: a valid HANDLE to a service or SCM database
+
+        :return: On error it raises an exception. Otherwise it was successful
+        """
         closeHandle = SVCCTLRCloseServiceHandle()
         closeHandle['ContextHandle'] = handle
         ans = self.doRequest(closeHandle, checkReturn = 1)
         return SVCCTLRCloseServiceHandlerResponse(ans)
  
     def EnumServicesStatusW(self, handle, serviceType = SERVICE_KERNEL_DRIVER | SERVICE_FILE_SYSTEM_DRIVER | SERVICE_WIN32_OWN_PROCESS | SERVICE_WIN32_SHARE_PROCESS | SERVICE_INTERACTIVE_PROCESS, serviceState = SERVICE_STATE_ALL ):
+        """
+        enumerates service records in the specified SCM database
+
+        :param HANDLE handle: a valid HANDLE to the SCM database (see OpenSCManagerW)
+        :param INT serviceType: a value that specifies what types of service records to enumerate. By default it request all services. For a list of serviceTypes see [MS-SCMR] 3.1.4.14
+        :param INT serviceState: A value that specifies the service records to enumerate based on the value of their current state. By default it request all states. For a list of serviceState see [MS-SCMR] 3.1.4.14
+
+        :return: Returns a list of services. Each item on that list is a dictionary with the following keys/types:
+        {
+            'ServiceName'       : UNICODE,
+            'DisplayName'       : UNICODE,
+            'ServiceType'       : int,
+            'CurrentState'      : int,
+            'ControlsAccepted'  : int,
+        }
+
+        On error it raises an exception
+
+        """
         enumServices = SVCCTLREnumServicesStatusW()
         enumServices['ContextHandle'] = handle
         enumServices['ServiceType']   = serviceType
@@ -987,8 +1091,89 @@ class DCERPCSvcCtl:
 
         return enumServicesList
         
+    def ChangeServiceConfigW(self, handle,  displayName = None, binaryPathName = None, serviceType = None, startType = None, serviceStartName = None, password = None):
+        """
+        changes a service's configuration parameters in the SCM database
 
+        :param HANDLE handle: a valid HANDLE to the service (see OpenServiceW)
+        :param UNICODE displayName: the new display name of the service. None if you don't want to change this value.
+        :param UNICODE binaryPathName: the new pathname for the binary to be executed when starting the service. None if you don't want to change this value
+        :param UNICODE serviceType: the new type of the service. None if you don't want to change this value. See service types within this file or [MS-SCMR] section 3.1.4.12.
+        :param INT startType: the new startType of the service. None if you don't want to change this value. See [MS-SCMR] section 3.1.4.11 for a list of possible values.
+        :param UNICODE startStartName: the name of the account under which the service should run. None if you don't want to change this value. 
+        :param BINARY password: a password value for the user. None if you don't want to change this value.
+        
+        :return: On error it raises an exception. Otherwise it was successful
+        
+        VERY IMPORTANT: If you dare to change the username and password, you need to 
+        take care of the following:
+        From [MS-SCMR], section 3.1.4.12
+
+        The server MUST treat the lpPassword as a clear-text password if the client 
+        is using RPC over TCP, ncacn_ip_tcp (as specified in [MS-RPCE]). 
+        See section 2.1.2 Client.
+        The server MUST treat the lpPassword as encrypted and decrypt it, if the 
+        client is using a RPC over NP, ncacn_np (as specified in [MS-RPCE]). 
+        The server MUST first retrieve a session key as specified in [MS-CIFS] 
+        (section 3.5.4.4). An RPC server application requests the session key of 
+        a client and then uses the routine as specified in [MS-LSAD] (section 5.1.2) 
+        to decrypt the password.
+
+        It's your reponsibility to fill out the right password data in the password 
+        parameter
+        """
+        changeConfig = SVCCTLRChangeServiceConfigW()
+        changeConfig['ContextHandle'] = handle
+
+        if startType is not None:
+            changeConfig['StartType'] = startType
+
+        if binaryPathName is not None:
+            changeConfig['BinaryPathName'] = ndrutils.NDRUniqueStringW()
+            changeConfig['BinaryPathName']['Data'] =  (binaryPathName+'\x00'.encode('utf-16le'))
+        else:
+            changeConfig['BinaryPathName'] = '\x00'*4
+
+
+        if displayName is not None:
+            changeConfig['DisplayName'] = ndrutils.NDRUniqueStringW()
+            changeConfig['DisplayName']['Data'] = (displayName+'\x00'.encode('utf-16le'))
+        else:
+            changeConfig['DisplayName'] = '\x00'*4
+
+        if serviceType is not None:
+            changeConfig['ServiceType'] = serviceType
+
+        if serviceStartName is not None:
+            if serviceStartName.find('\\') <= 0:
+                # Local user, we gotta append .\
+                serviceStartName = '.\\'.encode('utf-16le') + serviceStartName
+            changeConfig['ServiceStartName'] = ndrutils.NDRUniqueStringW()
+            changeConfig['ServiceStartName']['Data'] = (serviceStartName+'\x00'.encode('utf-16le'))
+        else:
+            changeConfig['ServiceStartName'] = '\x00'*4
+
+        if password is not None:
+            data = password
+            changeConfig['Password'] = pack('<L',random.randint(1,65535))
+            changeConfig['Password'] += pack('<L',len(password))
+            changeConfig['Password'] += data
+            changeConfig['PwSize'] = len(password)
+        else:
+            changeConfig['Password'] = '\x00'*4
+
+        ans = self.doRequest(changeConfig, checkReturn = 1)
+        return ans
+ 
     def QueryServiceStatus(self, handle):
+        """
+        returns the current status of the specified service
+
+        :param HANDLE handle: a valid HANDLE to the service (see OpenServiceW)
+
+        :return: returns an SVCCTLServiceStatus structure with the service status. Call dump() method to see its contents. On error it raises an exception
+
+        """
         queryStatus = SVCCTLRQueryServiceStatus()
         queryStatus['ContextHandle'] = handle
 
@@ -996,6 +1181,13 @@ class DCERPCSvcCtl:
         return SVCCTLServiceStatus(ans)
 
     def QueryServiceConfigW(self, handle):
+        """
+        returns the configuration parameters of the specified service
+
+        :param HANDLE handle: a valid HANDLE to the service (see OpenServiceW)
+
+        :return: returns an SVCCTLRQueryServiceConfigWResponse structure with the service's configuration. Call dump() method to see its contents. On error it raises an exception
+        """
         class configStrings(Structure):
             structure = (
                 ('BinaryPathName',':',ndrutils.NDRStringW),
