@@ -4,7 +4,7 @@
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: transport.py 727 2012-10-07 15:32:34Z bethus@gmail.com $
+# $Id: transport.py 876 2013-10-29 16:04:19Z bethus@gmail.com $
 #
 # Author: Alberto Solino
 #
@@ -194,12 +194,21 @@ class DCERPCTransport:
                self._lmhash = binascii.a2b_hex(lmhash)
                self._nthash = binascii.a2b_hex(nthash)
             except:
+               self._lmhash = lmhash
+               self._nthash = nthash
                pass
 
     def doesSupportNTLMv2(self):
         # By default we'll be returning the library's deafult. Only on SMB Transports we might be able to know it beforehand
         return ntlm.USE_NTLMv2
 
+    def get_dce_rpc(self):
+        if isinstance(self, UDPTransport):
+            return dcerpc_v4.DCERPC_v4(self)
+        else:
+            return dcerpc.DCERPC_v5(self)
+
+
 class UDPTransport(DCERPCTransport):
     "Implementation of ncadg_ip_udp protocol sequence"
 
@@ -321,6 +330,8 @@ class SMBTransport(DCERPCTransport):
             self.__existing_smb = False
         else:
             self.__existing_smb = True
+            user, passwd, domain, lm, nt = smb_connection.getCredentials()
+            self.set_credentials(user, passwd, domain, lm, nt)
 
         self.__prefDialect = None
 
@@ -394,6 +405,12 @@ class SMBTransport(DCERPCTransport):
     def get_smb_connection(self):
         return self.__smb_connection
     
+    def set_smb_connection(self, smb_connection):
+        self.__smb_connection = smb_connection
+        user, passwd, domain, lm, nt = smb_connection.getCredentials()
+        self.set_credentials(user, passwd, domain, lm, nt)
+        self.__existing_smb = True
+
     def get_smb_server(self):
         # Raw Access to the SMBServer (whatever type it is)
         return self.__smb_connection.getSMBServer()
