@@ -4,7 +4,7 @@
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: ndr.py 1109 2014-01-21 19:44:16Z bethus@gmail.com $
+# $Id: ndr.py 1186 2014-04-01 18:58:56Z bethus@gmail.com $
 #
 # [C706] Transfer NDR Syntax implementation
 # 
@@ -365,22 +365,26 @@ class NDR(object):
         return self
 
     def fromStringReferents(self, data, soFar = 0):
+        soFar0 = soFar
         for fieldName, fieldTypeOrClass in self.commonHdr+self.structure:
             if isinstance(self.fields[fieldName], NDR):
-                data = self.fields[fieldName].fromStringReferents(data, soFar)
-                soFar += len(self.fields[fieldName].getDataReferents(soFar))
-                data = self.fields[fieldName].fromStringReferent(data, soFar)
-                soFar += len(self.fields[fieldName].getDataReferent(soFar))
-        return data
+                nSoFar = self.fields[fieldName].fromStringReferents(data, soFar)
+                soFar += nSoFar
+                nSoFar2 = self.fields[fieldName].fromStringReferent(data[nSoFar:], soFar)
+                soFar += nSoFar2
+                data = data[nSoFar+nSoFar2:]
+        return soFar - soFar0
 
     def fromStringReferent(self, data, soFar = 0):
         if hasattr(self, 'referent') is not True:
-            return data
+            return 0
+
         soFar0 = soFar
+
         if self.fields.has_key('ReferentID'):
             if self['ReferentID'] == 0:
                 # NULL Pointer, there's no referent for it
-                return data
+                return 0
 
         for fieldName, fieldTypeOrClass in self.referent:
             size = self.calcUnPackSize(fieldTypeOrClass, data)
@@ -396,13 +400,13 @@ class NDR(object):
 
             if isinstance(self.fields[fieldName], NDR):
                 size = len(self.fields[fieldName].getData(soFar))
-                res = self.fields[fieldName].fromStringReferents(data[size:], soFar+size)
-                self.fields[fieldName].fromStringReferent(data[size:], soFar + size)
-                size += len(self.fields[fieldName].getDataReferents(soFar+size))+ len(self.fields[fieldName].getDataReferent(soFar+size)) 
+                nSoFar = self.fields[fieldName].fromStringReferents(data[size:], soFar + size)
+                nSoFar2 = self.fields[fieldName].fromStringReferent(data[size + nSoFar:], soFar + size + nSoFar)
+                size += nSoFar + nSoFar2 
             data = data[size:]
             soFar += size
 
-        return data 
+        return soFar-soFar0
 
     def pack(self, fieldName, fieldTypeOrClass, soFar = 0):
         if self.debug:
@@ -531,12 +535,13 @@ class NDR(object):
                     if pad > 0:
                         soFarItems += pad
                         data = data[pad:]
-                    data = itemn.fromStringReferents(data, soFarItems+soFar)
-                    soFarItems += len(itemn.getDataReferents(soFarItems+soFar))
-                    data = itemn.fromStringReferent(data, soFarItems+soFar)
-                    soFarItems += len(itemn.getDataReferent(soFarItems+soFar))
+                    nSoFar = itemn.fromStringReferents(data, soFarItems+soFar)
+                    soFarItems += nSoFar
+                    data = data[nSoFar:]
+                    nSoFar2 = itemn.fromStringReferent(data, soFarItems+soFar)
+                    soFarItems += nSoFar2
+                    data = data[nSoFar2:]
                     answer2.append(itemn)
-                    numItems -= 1
                 answer = answer2
                 del(answer2)
 
@@ -712,9 +717,9 @@ class NDRCALL(NDR):
                     size = len(self.fields[fieldName].getData(soFar))
                     # Any referent information to unpack?
                     if isinstance(self.fields[fieldName], NDR):
-                        res = self.fields[fieldName].fromStringReferents(data[size:], soFar+size)
-                        self.fields[fieldName].fromStringReferent(data[size:], soFar + size + len(res))
-                    size+= len(data[size:]) - len(res)
+                        nSoFar = self.fields[fieldName].fromStringReferents(data[size:], soFar + size)
+                        nSoFar2 = self.fields[fieldName].fromStringReferent(data[size + nSoFar:], soFar + size + nSoFar)
+                        size += nSoFar + nSoFar2 
 
                 data = data[size:]
                 soFar += size
@@ -736,6 +741,9 @@ class NDRCALL(NDR):
 
         return self
 
+# Top Level Struct == NDRCALL 
+NDRTLSTRUCT = NDRCALL
+
 # NDR Primitives
 
 class NDRSMALL(NDR):
@@ -851,7 +859,7 @@ class NDRENUM(NDR):
         if msg != '':
             print msg,
 
-        print " %s" % self.enumItems(self.fields['Data']).name
+        print " %s" % self.enumItems(self.fields['Data']).name,
 
 # NDR Constructed Types (arrays, strings, structures, unions, variant structures, pipes and pointers)
 
@@ -1394,7 +1402,17 @@ class NDRUNION(NDR):
                 self.__init__(None, isNDR64=self._isNDR64, topLevel = self.topLevel)
                 self.fields['tag']['Data'] = value
             else:
-                raise Exception("Unknown tag %d for union!" % value)
+                # Let's see if we have a default value
+                if self.union.has_key('default'):
+                    if self.union['default'] is None:
+                        self.structure = ()
+                    else:
+                        self.structure = (self.union['default']),
+                        # Init again the structure
+                        self.__init__(None, isNDR64=self._isNDR64, topLevel = self.topLevel)
+                    self.fields['tag']['Data'] = 0xffff
+                else:
+                    raise Exception("Unknown tag %d for union!" % value)
         else:
             return NDR.__setitem__(self,key,value)
 
@@ -1424,17 +1442,24 @@ class NDRUNION(NDR):
         # Now we need to align what's coming next.
         # This doesn't come from the documentation but from seeing the packets in the wire
         # for some reason, even if the next field is a SHORT, it should be aligned to
-        # a DWORD, or HYPER if NDR64
+        # a DWORD, or HYPER if NDR64. 
         if self._isNDR64:
             align = 8
         else:
-            align = 4
+            if hasattr(self, 'notAlign'):
+                align = 1
+            else:
+                align = 4
 
         pad = (align - (soFar % align)) % align
         if pad > 0:
             data = data + '\xbb'*pad
             soFar += pad
 
+        if self.structure is ():
+            self.data = data
+            return data
+
         for fieldName, fieldTypeOrClass in self.structure:
             try:
                 pad = self.calculatePad(fieldName, fieldTypeOrClass, data, soFar, packing = True)
@@ -1467,7 +1492,17 @@ class NDRUNION(NDR):
                 self.structure = (self.union[tag]),
                 self.__init__(None, isNDR64=self._isNDR64, topLevel = self.topLevel)
             else:
-                raise Exception("Unknown tag %d for union!" % tag)
+                # Let's see if we have a default value
+                if self.union.has_key('default'):
+                    if self.union['default'] is None:
+                        self.structure = ()
+                    else:
+                        self.structure = (self.union['default']),
+                        # Init again the structure
+                        self.__init__(None, isNDR64=self._isNDR64, topLevel = self.topLevel)
+                    self.fields['tag']['Data'] = 0xffff
+                else:
+                    raise Exception("Unknown tag %d for union!" % tag)
 
         if self.rawData is None:
             self.rawData = data
@@ -1493,17 +1528,23 @@ class NDRUNION(NDR):
         # Now we need to align what's coming next.
         # This doesn't come from the documentation but from seeing the packets in the wire
         # for some reason, even if the next field is a SHORT, it should be aligned to
-        # a DWORD, or HYPER if NDR64
+        # a DWORD, or HYPER if NDR64. 
         if self._isNDR64:
             align = 8
         else:
-            align = 4
+            if hasattr(self, 'notAlign'):
+                align = 1
+            else:
+                align = 4
 
         pad = (align - (soFar % align)) % align
         if pad > 0:
             data = data[pad:]
             soFar += pad
 
+        if self.structure is ():
+            return self
+
         for fieldName, fieldTypeOrClass in self.structure:
             size = self.calcUnPackSize(fieldTypeOrClass, data)
             pad = self.calculatePad(fieldName, fieldTypeOrClass, data, soFar = soFar, packing = False)
