@@ -1,12 +1,17 @@
-# Copyright (c) 2003-2012 CORE Security Technologies
+# Copyright (c) 2003-2014 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: rpcrt.py 1078 2014-01-10 13:48:41Z bethus@gmail.com $
+# $Id: rpcrt.py 1244 2014-06-26 13:54:47Z bethus@gmail.com $
 #
-# Partial C706.pdf + [MS-RPCE] implementation
+# Description:
+#   Partial C706.pdf + [MS-RPCE] implementation
+#
+#   Best way to learn how to use these calls is to grab the protocol standard
+#   so you understand what the call does, and then read the test case located
+#   at https://code.google.com/p/impacket/source/browse/#svn%2Ftrunk%2Fimpacket%2Ftestcases%2FSMB_RPC
 #
 # ToDo: 
 # [ ] Take out all the security provider stuff out of here (e.g. RPC_C_AUTHN_WINNT)
@@ -14,6 +19,8 @@
 #     more SSP (e.g. NETLOGON)
 # 
 
+import logging
+import socket
 from binascii import a2b_hex
 from Crypto.Cipher import ARC4
 
@@ -22,6 +29,10 @@ from impacket import ntlm
 from impacket.structure import Structure,pack,unpack
 from impacket import uuid
 from impacket.uuid import uuidtup_to_bin, generate, stringver_to_bin, bin_to_uuidtup
+from impacket.dcerpc.v5.dtypes import UCHAR, ULONG, USHORT
+from impacket.dcerpc.v5.ndr import NDRSTRUCT
+from impacket import hresult_errors
+from threading import Thread
 
 # MS/RPC Constants
 MSRPC_REQUEST   = 0x00
@@ -589,15 +600,15 @@ class MSRPCHeader(Structure):
         ('type','B=0'),                                   # 2
         ('flags','B=0'),                                  # 3
         ('representation','<L=0x10'),                     # 4
-        ('frag_len','<H=self._SIZE+len(pduData)+len(pad)+len(sec_trailer)+len(auth_data)'),  # 8
+        ('frag_len','<H=self._SIZE+len(auth_data)+(16 if (self["flags"] & 0x80) > 0 else 0)+len(pduData)+len(pad)+len(sec_trailer)'),  # 8
         ('auth_len','<H=len(auth_data)'),                 # 10
         ('call_id','<L=1'),                               # 12    <-- Common up to here (including this)
     )
 
     structure = ( 
-        ('dataLen','_-pduData','self["frag_len"]-self["auth_len"]-self._SIZE-(8 if self["auth_len"] > 0 else 0)'),  
+        ('dataLen','_-pduData','self["frag_len"]-self["auth_len"]-self._SIZE-(8 if self["auth_len"] > 0 else 0)'),
         ('pduData',':'),                                
-        ('_pad', '_-pad','(4 - ((self._SIZE + len(self["pduData"])) & 3) & 3)'),
+        ('_pad', '_-pad','(4 - ((self._SIZE + (16 if (self["flags"] & 0x80) > 0 else 0) + len(self["pduData"])) & 3) & 3)'),
         ('pad', ':'),
         ('_sec_trailer', '_-sec_trailer', '8 if self["auth_len"] > 0 else 0'),
         ('sec_trailer',':'),
@@ -620,7 +631,7 @@ class MSRPCHeader(Structure):
             self['pad'] = ''
 
     def get_header_size(self):
-        return self._SIZE
+        return self._SIZE + (16 if (self["flags"] & MSRPC_NOUUID) > 0 else 0)
 
     def get_packet(self):
         if self['auth_data'] != '':
@@ -638,6 +649,8 @@ class MSRPCRequestHeader(MSRPCHeader):
         ('alloc_hint','<L=0'),                            # 16
         ('ctx_id','<H=0'),                                # 20
         ('op_num','<H=0'),                                # 22
+        ('_uuid','_-uuid','16 if self["flags"] & 0x80 > 0 else 0' ), # 22
+        ('uuid',':'),                                # 22
     )
 
     def __init__(self, data = None, alignment = 0):
@@ -645,6 +658,7 @@ class MSRPCRequestHeader(MSRPCHeader):
         if data is None:
            self['type'] = MSRPC_REQUEST
            self['ctx_id'] = 0
+           self['uuid'] = ''
 
 class MSRPCRespHeader(MSRPCHeader):
     _SIZE = 24
@@ -749,7 +763,6 @@ class MSRPCBindNak(Structure):
             self['SupportedVersions'] = ''
 
 class DCERPC:
-    _max_ctx = 0
     # Standard NDR Representation
     NDRSyntax   = uuidtup_to_bin(('8a885d04-1ceb-11c9-9fe8-08002b104860', '2.0'))
     # NDR 64
@@ -762,6 +775,9 @@ class DCERPC:
         self._max_frag = None
         self.set_default_max_fragment_size()
 
+    def get_rpc_transport(self):
+        return self._transport
+
     def set_ctx_id(self, ctx_id):
         self._ctx = ctx_id
 
@@ -792,38 +808,46 @@ class DCERPC:
     def set_auth_type(self, auth_type, callback = None): pass
     def get_idempotent(self): return 0
     def set_idempotent(self, flag): pass
-    def call(self, function, body):
-        return self.send(DCERPC_RawCall(function, str(body)))
-    def request(self, request):
+    def call(self, function, body, uuid=None):
+        return self.send(DCERPC_RawCall(function, str(body), uuid))
+    def request(self, request, uuid=None):
         if self.transfer_syntax == self.NDR64Syntax:
             request.changeTransferSyntax(self.NDR64Syntax)
             isNDR64 = True
         else:
             isNDR64 = False
 
-        self.call(request.opnum, request)
+        self.call(request.opnum, request, uuid)
         answer = self.recv()
-        respClass = request.__module__ + '.' + request.__class__.__name__ + 'Response'
+        try:
+            module = __import__(request.__module__.split('.')[-1], globals(), locals(), -1)
+        except:
+            # Try the subdirectories
+            module = __import__('%s.%s' % (request.__module__.split('.')[-2],request.__module__.split('.')[-1]) , globals(), locals(), -1)
+        respClass = getattr(module, request.__class__.__name__+'Response')
         if  answer[-4:] != '\x00\x00\x00\x00':
             error_code = unpack('<L', answer[-4:])[0]
             if rpc_status_codes.has_key(error_code):
                 # This is an error we can handle
                 exception = DCERPCException(error_code)
             else:    
-                sessionErrorClass = request.__module__ + '.DCERPCSessionError'
+                sessionErrorClass = getattr(module, 'DCERPCSessionError')
                 try: 
                     # Try to unpack the answer, even if it is an error, it works most of the times
-                    response =  eval(respClass)(answer, isNDR64 = isNDR64)
+                    response =  respClass(answer, isNDR64 = isNDR64)
                 except:
                     # No luck :(
-                    exception = eval(sessionErrorClass)(error_code = error_code)
+                    exception = sessionErrorClass(error_code = error_code)
                 else:
-                    exception = eval(sessionErrorClass)(response)
+                    exception = sessionErrorClass(response)
             raise exception
         else:
-            response =  eval(respClass)(answer, isNDR64 = isNDR64)
+            response =  respClass(answer, isNDR64 = isNDR64)
             return response
 
+class DCERPC_v4(DCERPC):
+    pass
+
 class DCERPC_v5(DCERPC):
     def __init__(self, transport):
         DCERPC.__init__(self, transport)
@@ -864,6 +888,9 @@ class DCERPC_v5(DCERPC):
     def set_max_tfrag(self, size):
         self.__max_xmit_size = size
     
+    def get_credentials(self):
+        return self.__username, self.__password, self.__domain, self.__lmhash, self.__nthash
+
     def set_credentials(self, username, password, domain = '', lmhash = '', nthash = ''):
         self.set_auth_level(RPC_C_AUTHN_LEVEL_CONNECT)
         self.__username = username
@@ -1167,8 +1194,15 @@ class DCERPC_v5(DCERPC):
                 status_code = unpack("<L",self.response_data[off:off+4])[0]
                 if rpc_status_codes.has_key(status_code):
                     raise Exception(rpc_status_codes[status_code])
+                elif rpc_status_codes.has_key(status_code & 0xffff):
+                    raise Exception(rpc_status_codes[status_code & 0xffff])
                 else:
-                    raise Exception('Unknown DCE RPC fault status code: %.8x' % status_code)
+                    if hresult_errors.ERROR_MESSAGES.has_key(status_code):
+                        error_msg_short = hresult_errors.ERROR_MESSAGES[status_code][0]
+                        error_msg_verbose = hresult_errors.ERROR_MESSAGES[status_code][1] 
+                        raise Exception('%s - %s' % (error_msg_short, error_msg_verbose))
+                    else:
+                        raise Exception('Unknown DCE RPC fault status code: %.8x' % status_code)
 
             if self.response_header['flags'] & MSRPC_LASTFRAG:
                 # No need to reassembly DCERPC
@@ -1255,17 +1289,261 @@ class DCERPC_v5(DCERPC):
         answer.set_auth_type(self.__auth_type)
         answer.set_auth_level(self.__auth_level)
 
-        self._max_ctx += 1
-        answer.set_ctx_id(self._max_ctx)
+        answer.set_ctx_id(self._ctx+1)
         answer.__callid = self.__callid
         answer.bind(newUID, alter = 1, bogus_binds = bogus_binds, transfer_syntax = bin_to_uuidtup(self.transfer_syntax))
         return answer
 
 class DCERPC_RawCall(MSRPCRequestHeader):
-    def __init__(self, op_num, data = ''):
+    def __init__(self, op_num, data = '', uuid=None):
         MSRPCRequestHeader.__init__(self)
         self['op_num'] = op_num
         self['pduData'] = data
+        if uuid is not None:
+            self['flags'] |= MSRPC_NOUUID
+            self['uuid'] = uuid
 
     def setData(self, data):
         self['pduData'] = data
+
+# 2.2.6 Type Serialization Version 1
+class CommonHeader(NDRSTRUCT):
+    structure = (
+        ('Version', UCHAR),
+        ('Endianness', UCHAR),
+        ('CommonHeaderLength', USHORT),
+        ('Filler', ULONG),
+    )
+    def __init__(self, data = None,isNDR64 = False):
+        NDRSTRUCT.__init__(self, data, isNDR64)
+        if data is None:
+            self['Version'] = 1
+            self['Endianness'] = 0x10
+            self['CommonHeaderLength'] = 8
+            self['Filler'] = 0xcccccccc
+
+class PrivateHeader(NDRSTRUCT):
+    structure = (
+        ('ObjectBufferLength', ULONG),
+        ('Filler', ULONG),
+    )
+    def __init__(self, data = None,isNDR64 = False):
+        NDRSTRUCT.__init__(self, data, isNDR64)
+        if data is None:
+            self['Filler'] = 0xcccccccc
+
+class TypeSerialization1(NDRSTRUCT):
+    commonHdr = (
+        ('CommonHeader', CommonHeader),
+        ('PrivateHeader', PrivateHeader),
+    )
+    def getData(self, soFar = 0):
+        self['PrivateHeader']['ObjectBufferLength'] = len(NDRSTRUCT.getData(self, soFar))+len(NDRSTRUCT.getDataReferents(self, soFar))-len(self['CommonHeader'])-len(self['PrivateHeader'])
+        return NDRSTRUCT.getData(self, soFar)
+
+class DCERPCServer(Thread):
+    """
+    A minimalistic DCERPC Server, mainly used by the smbserver, for now. Might be useful
+    for other purposes in the future, but we should do it way stronger.
+    If you want to implement a DCE Interface Server, use this class as the base class
+    """
+    def __init__(self):
+        Thread.__init__(self)
+        self._listenPort    = 0
+        self._listenAddress = '127.0.0.1'
+        self._listenUUIDS   = {}
+        self._boundUUID     = ''
+        self._sock          = None
+        self._clientSock    = None
+        self._callid        = 1
+        self._max_frag       = None
+        self._max_xmit_size = 4280
+        self.__log = logging.getLogger()
+        self._sock = socket.socket()
+        self._sock.bind((self._listenAddress,self._listenPort))
+
+    def log(self, msg, level=logging.INFO):
+        self.__log.log(level,msg)
+
+    def addCallbacks(self, UUID, secondaryAddr, callbacks):
+        """
+        adds a call back to a UUID/opnum call
+        
+        :param uuid UUID: the interface UUID
+        :param string secondaryAddr: the secondary address to answer as part of the bind request (e.g. \\\\PIPE\\\\srvsvc)
+        :param dict callbacks: the callbacks for each opnum. Format is [opnum] = callback
+        """
+        self._listenUUIDS[uuidtup_to_bin(UUID)] = {}
+        self._listenUUIDS[uuidtup_to_bin(UUID)]['SecondaryAddr'] = secondaryAddr
+        self._listenUUIDS[uuidtup_to_bin(UUID)]['CallBacks'] = callbacks
+        self.log("Callback added for UUID %s V:%s" % UUID)
+
+    def setListenPort(self, portNum):
+        self._listenPort = portNum
+        self._sock = socket.socket()
+        self._sock.bind((self._listenAddress,self._listenPort))
+
+    def getListenPort(self):
+        return self._sock.getsockname()[1]
+
+    def recv(self):
+        finished = False
+        forceRecv = 0
+        retAnswer = ''
+        while not finished:
+            # At least give me the MSRPCRespHeader, especially important for TCP/UDP Transports
+            self.response_data = self._clientSock.recv(MSRPCRespHeader._SIZE)
+            # No data?, connection might have closed
+            if self.response_data == '':
+                return None
+            self.response_header = MSRPCRespHeader(self.response_data)
+            # Ok, there might be situation, especially with large packets, 
+            # that the transport layer didn't send us the full packet's contents
+            # So we gotta check we received it all
+            while ( len(self.response_data) < self.response_header['frag_len'] ):
+               self.response_data += self._clientSock.recv(self.response_header['frag_len']-len(self.response_data))
+            self.response_header = MSRPCRespHeader(self.response_data)
+            if self.response_header['flags'] & MSRPC_LASTFRAG:
+                # No need to reassembly DCERPC
+                finished = True
+            else:
+                # Forcing Read Recv, we need more packets!
+                forceRecv = 1
+            answer = self.response_header['pduData']
+            auth_len = self.response_header['auth_len']
+            if auth_len:
+                auth_len += 8
+                auth_data = answer[-auth_len:]
+                sec_trailer = SEC_TRAILER(data = auth_data)
+                answer = answer[:-auth_len]
+                if sec_trailer['auth_pad_len']:
+                    answer = answer[:-sec_trailer['auth_pad_len']]
+              
+            retAnswer += answer
+        return self.response_data
+    
+    def run(self):
+        self._sock.listen(10)
+        while True:
+            self._clientSock, address = self._sock.accept()
+            try:
+                while True:
+                    data = self.recv()
+                    if data is None:
+                        # No data.. connection closed
+                        break
+                    answer = self.processRequest(data)
+                    if answer != None:
+                        self.send(answer)
+            except Exception, e:
+                import traceback
+                print traceback.print_exc()
+                pass
+            self._clientSock.close()
+
+    def send(self, data):
+        max_frag       = self._max_frag
+        if len(data['pduData']) > self._max_xmit_size - 32:
+            max_frag   = self._max_xmit_size - 32    # XXX: 32 is a safe margin for auth data
+
+        if self._max_frag:
+            max_frag   = min(max_frag, self._max_frag)
+        if max_frag and len(data['pduData']) > 0:
+            packet     = data['pduData']
+            offset     = 0
+            while 1:
+                toSend = packet[offset:offset+max_frag]
+                if not toSend:
+                    break
+                flags  = 0
+                if offset == 0:
+                    flags |= MSRPC_FIRSTFRAG
+                offset += len(toSend)
+                if offset == len(packet):
+                    flags |= MSRPC_LASTFRAG
+                data['flags']   = flags
+                data['pduData'] = toSend
+                self._clientSock.send(data.get_packet())
+        else:
+            self._clientSock.send(data.get_packet())
+        self._callid += 1
+
+    def bind(self,packet, bind):
+        # Standard NDR Representation
+        NDRSyntax   = ('8a885d04-1ceb-11c9-9fe8-08002b104860', '2.0')
+        resp = MSRPCBindAck()
+
+        resp['type']             = MSRPC_BINDACK
+        resp['flags']            = packet['flags']
+        resp['frag_len']         = 0
+        resp['auth_len']         = 0
+        resp['auth_data']        = ''
+        resp['call_id']          = packet['call_id'] 
+        resp['max_tfrag']        = bind['max_tfrag']
+        resp['max_rfrag']        = bind['max_rfrag']
+        resp['assoc_group']      = 0x1234
+        resp['ctx_num']          = 0
+
+        data      = bind['ctx_items']
+        ctx_items = ''
+        for i in range(bind['ctx_num']):
+            result = MSRPC_CONT_RESULT_USER_REJECT
+            item   = CtxItem(data)
+            data   = data[len(item):]
+
+            # First we check the Transfer Syntax is NDR32, what we support
+            if item['TransferSyntax'] == uuidtup_to_bin(NDRSyntax):
+                # Now Check if the interface is what we listen
+                reason = 1 # Default, Abstract Syntax not supported
+                for i in self._listenUUIDS:
+                    if item['AbstractSyntax'] == i:
+                        # Match, we accept the bind request
+                        resp['SecondaryAddr']    = self._listenUUIDS[item['AbstractSyntax']]['SecondaryAddr']
+                        resp['SecondaryAddrLen'] = len(resp['SecondaryAddr'])+1
+                        reason           = 0
+                        self._boundUUID = i
+            else:
+                # Fail the bind request for this context
+                reason = 2 # Transfer Syntax not supported
+            if reason == 0:
+               result = MSRPC_CONT_RESULT_ACCEPT
+            resp['ctx_num']             += 1
+            itemResult                   = CtxItemResult()
+            itemResult['Result']         = result
+            itemResult['Reason']         = reason
+            itemResult['TransferSyntax'] = uuidtup_to_bin(NDRSyntax)
+            ctx_items                   += str(itemResult)
+
+        resp['Pad']              ='A'*((4-((resp["SecondaryAddrLen"]+MSRPCBindAck._SIZE) % 4))%4)
+        resp['ctx_items'] = ctx_items
+        resp['frag_len']  = len(str(resp))
+
+        self._clientSock.send(str(resp)) 
+        return None
+
+    def processRequest(self,data):
+        packet = MSRPCHeader(data)
+        if packet['type'] == MSRPC_BIND:
+            bind   = MSRPCBind(packet['pduData'])
+            packet = self.bind(packet, bind)
+        elif packet['type'] == MSRPC_REQUEST:
+            request          = MSRPCRequestHeader(data)
+            response         = MSRPCRespHeader(data)
+            response['type'] = MSRPC_RESPONSE
+            # Serve the opnum requested, if not, fails
+            if self._listenUUIDS[self._boundUUID]['CallBacks'].has_key(request['op_num']):
+                # Call the function 
+                returnData          = self._listenUUIDS[self._boundUUID]['CallBacks'][request['op_num']](request['pduData'])
+                response['pduData'] = returnData
+            else:
+                response['type']    = MSRPC_FAULT
+                response['pduData'] = struct.pack('<L',0x000006E4L)
+            response['frag_len'] = len(response)
+            return response
+        else:
+            # Defaults to a fault
+            packet         = MSRPCRespHeader(data)
+            packet['type'] = MSRPC_FAULT
+
+        return packet
+
