@@ -4,13 +4,22 @@
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: rrp.py 1105 2014-01-19 13:56:06Z bethus@gmail.com $
+# $Id: rrp.py 1244 2014-06-26 13:54:47Z bethus@gmail.com $
 #
 # Author: Alberto Solino
 #
 # Description:
 #   [MS-RRP] Interface implementation
 #
+#   Best way to learn how to use these calls is to grab the protocol standard
+#   so you understand what the call does, and then read the test case located
+#   at https://code.google.com/p/impacket/source/browse/#svn%2Ftrunk%2Fimpacket%2Ftestcases%2FSMB_RPC
+#
+#   Some calls have helper functions, which makes it even easier to use.
+#   They are located at the end of this file. 
+#   Helper functions start with "h"<name of the call>.
+#   There are test cases for them too. 
+#
 
 from impacket.dcerpc.v5 import ndr
 from impacket.dcerpc.v5.ndr import NDRCALL, NDR, NDRSTRUCT, NDRPOINTER, NDRUniConformantVaryingArray
@@ -816,9 +825,10 @@ def hBaseRegOpenKey(dce, hKey, lpSubKey, dwOptions=0x00000001, samDesired = MAXI
 def hBaseRegQueryInfoKey(dce, hKey):
     request = BaseRegQueryInfoKey()
     request['hKey'] = hKey
-    request['lpClassIn'] = NULL
     # Not the cleanest way, but oh well
+    # Plus, Windows XP needs MaximumCount also set
     request.fields['lpClassIn'].fields['MaximumLength'] = 1024
+    request.fields['lpClassIn'].fields['Data'].fields['Data'].fields['MaximumCount'] = 1024/2
     return dce.request(request)
 
 def hBaseRegQueryValue(dce, hKey, lpValueName):
