@@ -4,7 +4,7 @@
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: samr.py 1119 2014-01-27 14:24:59Z bethus@gmail.com $
+# $Id: samr.py 1244 2014-06-26 13:54:47Z bethus@gmail.com $
 #
 # Author: Alberto Solino
 #
@@ -13,7 +13,7 @@
 #
 #   Best way to learn how to use these calls is to grab the protocol standard
 #   so you understand what the call does, and then read the test case located
-#   at https://code.google.com/p/impacket/source/browse/#svn%2Ftrunk%2Fimpacket%2Ftestcases%2FSMB-RPC
+#   at https://code.google.com/p/impacket/source/browse/#svn%2Ftrunk%2Fimpacket%2Ftestcases%2FSMB_RPC
 #
 #   Some calls have helper functions, which makes it even easier to use.
 #   They are located at the end of this file. 
@@ -253,7 +253,7 @@ DOMAIN_USER_RID_GUEST                 = 0x000001F5
 DOMAIN_USER_RID_KRBTGT                = 0x000001F6
 DOMAIN_GROUP_RID_USERS                = 0x00000201
 DOMAIN_GROUP_RID_COMPUTERS            = 0x00000203
-DOMAIN_GROUP_RID_CONTROLLERS          = 0x00000203
+DOMAIN_GROUP_RID_CONTROLLERS          = 0x00000204
 DOMAIN_ALIAS_RID_ADMINS               = 0x00000220
 DOMAIN_GROUP_RID_READONLY_CONTROLLERS = 0x00000209
 
@@ -276,6 +276,16 @@ SAM_VALIDATE_PASSWORD_HISTORY        = 0x00000020
 ################################################################################
 # STRUCTURES
 ################################################################################
+class RPC_UNICODE_STRING_ARRAY(NDRUniConformantVaryingArray):
+    item = RPC_UNICODE_STRING
+
+class RPC_UNICODE_STRING_ARRAY_C(NDRUniConformantArray):
+    item = RPC_UNICODE_STRING
+
+class PRPC_UNICODE_STRING_ARRAY(NDRPOINTER):
+    referent = (
+        ('Data',RPC_UNICODE_STRING_ARRAY_C),
+    )
 
 # 2.2.2.1 RPC_STRING, PRPC_STRING
 class RPC_STRING(NDRSTRUCT):
@@ -373,13 +383,15 @@ PENCRYPTED_NT_OWF_PASSWORD = PENCRYPTED_LM_OWF_PASSWORD
 #    item = '<L'
 class ULONG_ARRAY(NDRUniConformantArray):
     item = ULONG
-    pass
 
 class PULONG_ARRAY(NDRPOINTER):
     referent = (
         ('Data', ULONG_ARRAY),
     )
 
+class ULONG_ARRAY_CV(NDRUniConformantVaryingArray):
+    item = ULONG
+
 class SAMPR_ULONG_ARRAY(NDRSTRUCT):
     structure = (
         ('Count', ULONG),
@@ -419,6 +431,13 @@ class SAMPR_PSID_ARRAY_OUT(NDRSTRUCT):
         ('Sids', PSAMPR_SID_INFORMATION_ARRAY),
     )
 
+# 2.2.3.8 SAMPR_RETURNED_USTRING_ARRAY
+class SAMPR_RETURNED_USTRING_ARRAY(NDRSTRUCT):
+    structure = (
+        ('Count', ULONG),
+        ('Element', PRPC_UNICODE_STRING_ARRAY),
+    )
+
 # 2.2.3.9 SAMPR_RID_ENUMERATION
 class SAMPR_RID_ENUMERATION(NDRSTRUCT):
     structure = (
@@ -1350,9 +1369,6 @@ class PSAM_VALIDATE_OUTPUT_ARG(NDRPOINTER):
         ('Data', SAM_VALIDATE_OUTPUT_ARG),
     )
 
-class RPC_UNICODE_STRING_ARRAY(NDRUniConformantVaryingArray):
-    item = RPC_UNICODE_STRING
-
 ################################################################################
 # RPC CALLS
 ################################################################################
@@ -1596,7 +1612,7 @@ class SamrLookupNamesInDomain(NDRCALL):
 
 class SamrLookupNamesInDomainResponse(NDRCALL):
     structure = (
-       ('RelativeIds',ULONG),
+       ('RelativeIds',SAMPR_ULONG_ARRAY),
        ('Use',SAMPR_ULONG_ARRAY),
        ('ErrorCode',ULONG),
     )
@@ -1606,13 +1622,12 @@ class SamrLookupIdsInDomain(NDRCALL):
     structure = (
        ('DomainHandle',SAMPR_HANDLE),
        ('Count',ULONG),
-       ('RelativeIds',SAMPR_ULONG_ARRAY),
+       ('RelativeIds',ULONG_ARRAY_CV),
     )
 
 class SamrLookupIdsInDomainResponse(NDRCALL):
     structure = (
-       #('Names',PSAMPR_RETURNED_USTRING_ARRAY),
-       ('Names',RPC_UNICODE_STRING_ARRAY),
+       ('Names',SAMPR_RETURNED_USTRING_ARRAY),
        ('Use',SAMPR_ULONG_ARRAY),
        ('ErrorCode',ULONG),
     )
@@ -2355,7 +2370,7 @@ def hSamrEnumerateAliasesInDomain(dce, domainHandle, enumerationContext=0, prefe
 def hSamrEnumerateUsersInDomain(dce, domainHandle, userAccountControl=USER_NORMAL_ACCOUNT, enumerationContext=0, preferedMaximumLength=0xffffffff):
     request = SamrEnumerateUsersInDomain()
     request['DomainHandle'] = domainHandle
-    request['UserAccountControl'] = enumerationContext
+    request['UserAccountControl'] = userAccountControl
     request['EnumerationContext'] = enumerationContext
     request['PreferedMaximumLength'] = preferedMaximumLength
     return dce.request(request)
@@ -2704,3 +2719,31 @@ def hSamrValidatePassword(dce, inputArg):
     request['InputArg'] = inputArg
     return dce.request(request)
 
+def hSamrLookupNamesInDomain(dce, domainHandle, names):
+    request = SamrLookupNamesInDomain()
+    request['DomainHandle'] =  domainHandle
+    request['Count'] = len(names)
+    for name in names:
+        entry = RPC_UNICODE_STRING()
+        entry['Data'] = name
+        request['Names'].append(entry)
+
+    request.fields['Names'].fields['MaximumCount'] = 1000
+    request.dump()
+
+    return dce.request(request)
+
+def hSamrLookupIdsInDomain(dce, domainHandle, ids):
+    request = SamrLookupIdsInDomain()
+    request['DomainHandle'] =  domainHandle
+    request['Count'] = len(ids)
+    for id in ids:
+        entry = ULONG()
+        entry['Data'] = id
+        request['RelativeIds'].append(entry)
+
+    request.fields['RelativeIds'].fields['MaximumCount'] = 1000
+    request.dump()
+
+    return dce.request(request)
+
