@@ -4,7 +4,7 @@
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: srvs.py 1106 2014-01-19 14:17:01Z bethus@gmail.com $
+# $Id: srvs.py 1244 2014-06-26 13:54:47Z bethus@gmail.com $
 #
 # Author: Alberto Solino
 #
@@ -13,7 +13,7 @@
 #
 #   Best way to learn how to use these calls is to grab the protocol standard
 #   so you understand what the call does, and then read the test case located
-#   at https://code.google.com/p/impacket/source/browse/#svn%2Ftrunk%2Fimpacket%2Ftestcases%2FSMB-RPC
+#   at https://code.google.com/p/impacket/source/browse/#svn%2Ftrunk%2Fimpacket%2Ftestcases%2FSMB_RPC
 #
 #   Some calls have helper functions, which makes it even easier to use.
 #   They are located at the end of this file. 
@@ -2495,7 +2495,6 @@ class NetrServerGetInfo(NDRCALL):
     structure = (
        ('ServerName', PSRVSVC_HANDLE),
        ('Level', DWORD),
-       ('InfoStruct', SERVER_INFO),
     )
 
 class NetrServerGetInfoResponse(NDRCALL):
@@ -3163,8 +3162,6 @@ def hNetrServerGetInfo(dce, level):
     request = NetrServerGetInfo()
     request['ServerName'] = NULL
     request['Level'] = level
-    request['InfoStruct']['tag'] = level
-    request['InfoStruct']['ServerInfo%d'%level] = NULL
     return dce.request(request)
 
 def hNetrServerDiskEnum(dce, level, resumeHandle = 0, preferedMaximumLength = 0xffffffff):
