@@ -5,7 +5,7 @@
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: ese.py 1124 2014-01-27 19:38:32Z bethus@gmail.com $
+# $Id: ese.py 1217 2014-04-16 20:33:05Z bethus@gmail.com $
 #
 # Description:
 #             Microsoft Extensive Storage Engine parser, just focused on trying 
@@ -28,7 +28,10 @@ import logging
 try:
     from collections import OrderedDict
 except:
-    from ordereddict.ordereddict import OrderedDict
+    try:
+        from ordereddict.ordereddict import OrderedDict
+    except:
+        from ordereddict import OrderedDict
 from impacket import structure, version
 from impacket.structure import Structure
 from struct import unpack
