@@ -4,7 +4,7 @@
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: serviceinstall.py 1074 2014-01-09 18:46:43Z bethus@gmail.com $
+# $Id: serviceinstall.py 1141 2014-02-12 16:39:51Z bethus@gmail.com $
 #
 # Service Install Helper library used by psexec and smbrelayx
 # You provide an already established connection and an exefile 
@@ -98,7 +98,7 @@ class ServiceInstall():
             resp = self.rpcsvc.OpenSCManagerW()
         except:
             print "[!] Error opening SVCManager on %s....." % self.connection.getRemoteHost()
-            return 0
+            raise Exception('Unable to open SVCManager')
         else:
             return resp['ContextHandle']
 
@@ -171,6 +171,7 @@ class ServiceInstall():
                             pass
                         self.rpcsvc.CloseServiceHandle(service)
                     self.rpcsvc.CloseServiceHandle(svcManager)
+                    return True
             except Exception, e:
                 print "[!] Error performing the installation, cleaning up: %s" %e
                 try:
@@ -187,6 +188,7 @@ class ServiceInstall():
                         self.rpcsvc.DeleteService(service)
                     except:
                         pass
+            return False
       
     def uninstall(self):
         fileCopied = True
