@@ -4,7 +4,7 @@
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: smb3.py 741 2013-04-25 22:07:46Z bethus@gmail.com $
+# $Id: smb3.py 1126 2014-01-27 20:22:33Z bethus@gmail.com $
 #
 # Author: Alberto Solino (beto@coresecurity.com)
 #
@@ -110,7 +110,7 @@ class SessionError(Exception):
 
 
 class SMB3:
-    def __init__(self, remote_name, remote_host, my_name = None, host_type = nmb.TYPE_SERVER, sess_port = 445, timeout=10, UDP = 0, preferredDialect = None, session = None):
+    def __init__(self, remote_name, remote_host, my_name = None, host_type = nmb.TYPE_SERVER, sess_port = 445, timeout=60, UDP = 0, preferredDialect = None, session = None):
 
         # [MS-SMB2] Section 3
         self.RequireMessageSigning = False    #
@@ -386,6 +386,14 @@ class SMB3:
                 self._Connection['ServerCapabilities'] = negResp['Capabilities']
                 self._Connection['ServerSecurityMode'] = negResp['SecurityMode']
 
+    def getCredentials(self):
+        return (
+            self.__userName,
+            self.__password,
+            self.__domain,
+            self.__lmhash,
+            self.__nthash)
+
     def login(self, user, password, domain = '', lmhash = '', nthash = ''):
         # If we have hashes, normalize them
         if ( lmhash != '' or nthash != ''):
@@ -397,6 +405,12 @@ class SMB3:
             except:
                 pass
 
+        self.__userName = user
+        self.__password = password
+        self.__domain   = domain
+        self.__lmhash   = lmhash
+        self.__nthash   = nthash
+       
         sessionSetup = SMB2SessionSetup()
         if self.RequireMessageSigning is True:
            sessionSetup['SecurityMode'] = SMB2_NEGOTIATE_SIGNING_REQUIRED
@@ -489,8 +503,8 @@ class SMB3:
                     # Calculate the key derivations for dialect 3.0
                     if self._Session['SigningRequired'] is True:
                         self._Session['SigningActivated'] = True
-                        if self._Connection['Dialect'] == SMB2_DIALECT_30:
-                            self._Session['ApplicationKey']  = crypto.KDF_CounterMode(exportedSessionKey, "SMB2APP\x00", "SmbRpc\x00", 128)
+                    if self._Connection['Dialect'] == SMB2_DIALECT_30:
+                        self._Session['ApplicationKey']  = crypto.KDF_CounterMode(exportedSessionKey, "SMB2APP\x00", "SmbRpc\x00", 128)
                     if self._Session['EncryptData'] is True:
                         self._Session['EncryptionKey']  = crypto.KDF_CounterMode(exportedSessionKey, "SMB2AESCCM\x00", "ServerIn \x00", 128)
                         self._Session['DecryptionKey']  = crypto.KDF_CounterMode(exportedSessionKey, "SMB2AESCCM\x00", "ServerOut\x00", 128)
@@ -1018,6 +1032,18 @@ class SMB3:
         if ans.isValidAnswer(STATUS_SUCCESS):
             return True
 
+    def getSessionKey(self):
+        if self.getDialect() == SMB2_DIALECT_30: 
+           return self._Session['ApplicationKey']
+        else:
+           return self._Session['SessionKey']
+
+    def setSessionKey(self, key):
+        if self.getDialect() == SMB2_DIALECT_30:
+           self._Session['ApplicationKey'] = key
+        else:
+           self._Session['SessionKey'] = key
+
     ######################################################################
     # Higher level functions
 
@@ -1079,12 +1105,12 @@ class SMB3:
             from impacket import smb
             while True:
                 try:
-                    res = self.queryDirectory( treeId, fileId, ntpath.basename(path), maxBufferSize = 65535 )
+                    res = self.queryDirectory( treeId, fileId, ntpath.basename(path), maxBufferSize = 65535, informationClass = FILE_FULL_DIRECTORY_INFORMATION )
                     nextOffset = 1
                     while nextOffset != 0:
-                        fileInfo = smb.SMBFindFileNamesInfo(smb.SMB.FLAGS2_UNICODE)
+                        fileInfo = smb.SMBFindFileFullDirectoryInfo(smb.SMB.FLAGS2_UNICODE)
                         fileInfo.fromString(res)
-                        files.append(smb.SharedFile(0,0,0,0,0,0,fileInfo['FileName'].decode('utf-16le'), fileInfo['FileName'].decode('utf-16le')))
+                        files.append(smb.SharedFile(fileInfo['CreationTime'],fileInfo['LastAccessTime'],fileInfo['LastChangeTime'],fileInfo['EndOfFile'],fileInfo['AllocationSize'],fileInfo['ExtFileAttributes'],fileInfo['FileName'].decode('utf-16le'), fileInfo['FileName'].decode('utf-16le')))
                         nextOffset = fileInfo['NextEntryOffset']
                         res = res[nextOffset:]
                 except SessionError, e:
@@ -1155,7 +1181,7 @@ class SMB3:
 
         return True
 
-    def retrieveFile(self, shareName, path, callback, mode = FILE_OPEN, offset = 0, password = None):
+    def retrieveFile(self, shareName, path, callback, mode = FILE_OPEN, offset = 0, password = None, shareAccessMode = FILE_SHARE_READ):
         # ToDo: Handle situations where share is password protected
         path = string.replace(path,'/', '\\')
         path = ntpath.normpath(path)
@@ -1166,7 +1192,7 @@ class SMB3:
         fileId = None
         from impacket import smb
         try:
-            fileId = self.create(treeId, path, FILE_READ_DATA, FILE_SHARE_READ, FILE_NON_DIRECTORY_FILE, mode, 0 )
+            fileId = self.create(treeId, path, FILE_READ_DATA, shareAccessMode, FILE_NON_DIRECTORY_FILE, mode, 0)
             res = self.queryInfo(treeId, fileId)
             fileInfo = smb.SMBQueryFileStandardInfo(res)
             fileSize = fileInfo['EndOfFile']
@@ -1188,7 +1214,7 @@ class SMB3:
                 self.close(treeId, fileId)
             self.disconnectTree(treeId) 
 
-    def storeFile(self, shareName, path, callback, mode = FILE_OVERWRITE_IF, offset = 0, password = None):
+    def storeFile(self, shareName, path, callback, mode = FILE_OVERWRITE_IF, offset = 0, password = None, shareAccessMode = FILE_SHARE_WRITE):
         # ToDo: Handle situations where share is password protected
         path = string.replace(path,'/', '\\')
         path = ntpath.normpath(path)
@@ -1198,7 +1224,7 @@ class SMB3:
         treeId = self.connectTree(shareName)
         fileId = None
         try:
-            fileId = self.create(treeId, path, FILE_WRITE_DATA, FILE_SHARE_WRITE, FILE_NON_DIRECTORY_FILE, mode, 0 )
+            fileId = self.create(treeId, path, FILE_WRITE_DATA, shareAccessMode, FILE_NON_DIRECTORY_FILE, mode, 0)
             finished = False
             writeOffset = offset
             while not finished:
