@@ -4,12 +4,12 @@
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: smbconnection.py 738 2013-04-09 18:40:15Z bethus@gmail.com $
+# $Id: smbconnection.py 1126 2014-01-27 20:22:33Z bethus@gmail.com $
 #
 # Author: Alberto Solino (beto@coresecurity.com)
 #
 # Description:
-#            
+#
 # Wrapper class for SMB1/2/3 so it's transparent for the client.
 # You can still play with the low level methods (version dependant)
 # by calling getSMBServer()
@@ -18,7 +18,7 @@
 import ntpath
 import string
 import socket
-from impacket import smb, smb3, nmb
+from impacket import smb, smb3, nmb, nt_errors
 from smb3structs import *
 
 # So the user doesn't need to import smb, the smb3 are already in here
@@ -37,7 +37,7 @@ class SMBConnection():
 
     :return: a SMBConnection instance, if not raises a SessionError exception
     """
-    def __init__(self, remoteName='', remoteHost='', myName = None, sess_port = 445, timeout=10, preferredDialect = None, existingConnection = None):
+    def __init__(self, remoteName='', remoteHost='', myName = None, sess_port = 445, timeout=60, preferredDialect = None, existingConnection = None):
 
         self._SMBConnection = 0
         self._dialect       = ''
@@ -50,7 +50,7 @@ class SMBConnection():
             self._SMBConnection = existingConnection
             return
 
-
+        ##preferredDialect = smb.SMB_DIALECT
         if preferredDialect is None:
             # If no preferredDialect sent, we try the highest available one.
             packet = self._negotiateSession(myName, remoteName, remoteHost, sess_port, timeout)
@@ -67,11 +67,11 @@ class SMBConnection():
                 self._SMBConnection = smb3.SMB3(remoteName, remoteHost, myName, hostType, sess_port, timeout, preferredDialect = preferredDialect)
             else:
                 print "Unknown dialect ", preferredDialect
-                raise 
+                raise
 
     def _negotiateSession(self, myName, remoteName, remoteHost, sess_port, timeout, extended_security = True):
-        # Here we follow [MS-SMB2] negotiation handshake trying to understand what dialects 
-        # (including SMB1) is supported on the other end. 
+        # Here we follow [MS-SMB2] negotiation handshake trying to understand what dialects
+        # (including SMB1) is supported on the other end.
 
         if not myName:
             myName = socket.gethostname()
@@ -79,7 +79,7 @@ class SMBConnection():
             if i > -1:
                 myName = myName[:i]
 
-        # If port 445 and the name sent is *SMBSERVER we're setting the name to the IP. This is to help some old applications still believing 
+        # If port 445 and the name sent is *SMBSERVER we're setting the name to the IP. This is to help some old applications still believing
         # *SMSBSERVER will work against modern OSes. If port is NETBIOS_SESSION_PORT the user better know about *SMBSERVER's limitations
         if sess_port == 445 and remoteName == '*SMBSERVER':
            remoteName = remoteHost
@@ -89,7 +89,7 @@ class SMBConnection():
         smbp = smb.NewSMBPacket()
         negSession = smb.SMBCommand(smb.SMB.SMB_COM_NEGOTIATE)
         if extended_security == True:
-            smbp['Flags2']=smb.SMB.FLAGS2_EXTENDED_SECURITY 
+            smbp['Flags2']=smb.SMB.FLAGS2_EXTENDED_SECURITY
         negSession['Data'] = '\x02NT LM 0.12\x00\x02SMB 2.002\x00\x02SMB 2.???\x00'
         smbp.addCommand(negSession)
         self._nmbSession.send_packet(str(smbp))
@@ -126,6 +126,9 @@ class SMBConnection():
     def isLoginRequired(self):
         return self._SMBConnection.is_login_required()
 
+    def getCredentials(self):
+        return self._SMBConnection.getCredentials()
+
     def login(self, user, password, domain = '', lmhash = '', nthash = ''):
         """
         logins into the target system
@@ -138,43 +141,78 @@ class SMBConnection():
 
         :return: None, raises a Session Error if error.
         """
-        return self._SMBConnection.login(user, password, domain, lmhash, nthash)
+        try: 
+            return self._SMBConnection.login(user, password, domain, lmhash, nthash)
+        except (smb.SessionError, smb3.SessionError), e:
+            raise SessionError(e.get_error_code())
 
     def isGuestSession(self):
-        return self._SMBConnection.isGuestSession()
+        try:
+            return self._SMBConnection.isGuestSession()
+        except (smb.SessionError, smb3.SessionError), e:
+            raise SessionError(e.get_error_code())
 
     def logoff(self):
-        return self._SMBConnection.logoff()
+        try:
+            return self._SMBConnection.logoff()
+        except (smb.SessionError, smb3.SessionError), e:
+            raise SessionError(e.get_error_code())
+
 
     def connectTree(self,share):
         if self.getDialect() == smb.SMB_DIALECT:
             share = ntpath.basename(share)
             share = '\\\\' + self.getRemoteHost() + '\\' + share
-        return self._SMBConnection.connect_tree(share)
+        try:
+            return self._SMBConnection.connect_tree(share)
+        except (smb.SessionError, smb3.SessionError), e:
+            raise SessionError(e.get_error_code())
+
 
     def disconnectTree(self, treeId):
-        return self._SMBConnection.disconnect_tree(treeId)
+        try:
+            return self._SMBConnection.disconnect_tree(treeId)
+        except (smb.SessionError, smb3.SessionError), e:
+            raise SessionError(e.get_error_code())
+
 
     def listShares(self):
+        """
+        get a list of available shares at the connected target
+
+        :return: a list containing dict entries for each share, raises exception if error
+        """
         # Get the shares through RPC
-        from impacket.dcerpc import transport, dcerpc, srvsvc
+        from impacket.dcerpc.v5 import transport, rpcrt, srvs
         rpctransport = transport.SMBTransport(self.getRemoteHost(), self.getRemoteHost(), filename = r'\srvsvc', smb_connection = self)
-        dce = dcerpc.DCERPC_v5(rpctransport)
+        dce = rpctransport.get_dce_rpc()
         dce.connect()
-        dce.bind(srvsvc.MSRPC_UUID_SRVSVC)
-        srv_svc = srvsvc.DCERPCSrvSvc(dce)
-        resp = srv_svc.get_share_enum_1(rpctransport.get_dip())
-        return resp
+        dce.bind(srvs.MSRPC_UUID_SRVS)
+        resp = srvs.hNetrShareEnum(dce, 1)
+        return resp['InfoStruct']['ShareInfo']['Level1']['Buffer']
 
     def listPath(self, shareName, path, password = None):
-        return self._SMBConnection.list_path(shareName, path, password)
+        """
+        list the files/directories under shareName/path
+
+        :param string shareName: a valid name for the share where the files/directories are going to be searched
+        :param string path: a base path relative to shareName
+
+        :return: a list containing smb.SharedFile items, raises a SessionError exception if error.
+        """
+
+        try:
+            return self._SMBConnection.list_path(shareName, path, password)
+        except (smb.SessionError, smb3.SessionError), e:
+            raise SessionError(e.get_error_code())
+
 
     def createFile(self, treeId, pathName, desiredAccess = GENERIC_ALL, shareMode = FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE, creationOption = FILE_NON_DIRECTORY_FILE, creationDisposition = FILE_OVERWRITE_IF , fileAttributes = FILE_ATTRIBUTE_NORMAL, impersonationLevel = SMB2_IL_IMPERSONATION, securityFlags = 0, oplockLevel = SMB2_OPLOCK_LEVEL_NONE, createContexts = None):
         """
         creates a remote file
 
-        :param HANDLE treeId: a valid handle for the share where the file is to be opened
-        :param string pathName: the path name to open
+        :param HANDLE treeId: a valid handle for the share where the file is to be created
+        :param string pathName: the path name of the file to create
         :return: a valid file descriptor, if not raises a SessionError exception.
         """
 
@@ -196,10 +234,17 @@ class SMBConnection():
 
             if createContexts is not None:
                 print "CreateContexts not supported in SMB1"
-      
-            return self._SMBConnection.nt_create_andx(treeId, pathName, cmd = ntCreate)
+
+            try:
+                return self._SMBConnection.nt_create_andx(treeId, pathName, cmd = ntCreate)
+            except (smb.SessionError, smb3.SessionError), e:
+                raise SessionError(e.get_error_code())
         else:
-            return self._SMBConnection.create(treeId, pathName, desiredAccess, shareMode, creationOption, creationDisposition, fileAttributes, impersonationLevel, securityFlags, oplockLevel, createContexts)
+            try:
+                return self._SMBConnection.create(treeId, pathName, desiredAccess, shareMode, creationOption, creationDisposition, fileAttributes, impersonationLevel, securityFlags, oplockLevel, createContexts)
+            except (smb.SessionError, smb3.SessionError), e:
+                raise SessionError(e.get_error_code())
+
 
     def openFile(self, treeId, pathName, desiredAccess = FILE_READ_DATA | FILE_WRITE_DATA, shareMode = FILE_SHARE_READ, creationOption = FILE_NON_DIRECTORY_FILE, creationDisposition = FILE_OPEN, fileAttributes = FILE_ATTRIBUTE_NORMAL, impersonationLevel = SMB2_IL_IMPERSONATION, securityFlags = 0, oplockLevel = SMB2_OPLOCK_LEVEL_NONE, createContexts = None):
         """
@@ -228,36 +273,49 @@ class SMBConnection():
 
             if createContexts is not None:
                 print "CreateContexts not supported in SMB1"
-      
-            return self._SMBConnection.nt_create_andx(treeId, pathName, cmd = ntCreate)
+
+            try:
+                return self._SMBConnection.nt_create_andx(treeId, pathName, cmd = ntCreate)
+            except (smb.SessionError, smb3.SessionError), e:
+                raise SessionError(e.get_error_code())
         else:
-            return self._SMBConnection.create(treeId, pathName, desiredAccess, shareMode, creationOption, creationDisposition, fileAttributes, impersonationLevel, securityFlags, oplockLevel, createContexts)
-        
+            try:
+                return self._SMBConnection.create(treeId, pathName, desiredAccess, shareMode, creationOption, creationDisposition, fileAttributes, impersonationLevel, securityFlags, oplockLevel, createContexts)
+            except (smb.SessionError, smb3.SessionError), e:
+                raise SessionError(e.get_error_code())
+
     def writeFile(self, treeId, fileId, data, offset=0):
         """
         writes data to a file
 
-        :param HANDLE treeId: a valid handle for the share where the file is to be opened
-        :param HANDLE fileId: a valid handle for the file/directory to be closed
+        :param HANDLE treeId: a valid handle for the share where the file is to be written
+        :param HANDLE fileId: a valid handle for the file
         :param string data: buffer with the data to write
         :param integer offset: offset where to start writing the data
 
         :return: amount of bytes written, if not raises a SessionError exception.
         """
-        return self._SMBConnection.writeFile(treeId, fileId, data, offset)
+        try:
+            return self._SMBConnection.writeFile(treeId, fileId, data, offset)
+        except (smb.SessionError, smb3.SessionError), e:
+            raise SessionError(e.get_error_code())
+
 
     def readFile(self, treeId, fileId, offset = 0, bytesToRead = None):
         """
         reads data from a file
 
-        :param HANDLE treeId: a valid handle for the share where the file is to be opened
-        :param HANDLE fileId: a valid handle for the file/directory to be closed
-        :param integer offset: offset where to start writing the data
+        :param HANDLE treeId: a valid handle for the share where the file is to be read
+        :param HANDLE fileId: a valid handle for the file to be read
+        :param integer offset: offset where to start reading the data
         :param integer bytesToRead: amount of bytes to read. If None, it will read Dialect['MaxBufferSize'] bytes.
 
         :return: the data read, if not raises a SessionError exception.
         """
-        return self._SMBConnection.read_andx(treeId, fileId, offset, bytesToRead)
+        try:
+            return self._SMBConnection.read_andx(treeId, fileId, offset, bytesToRead)
+        except (smb.SessionError, smb3.SessionError), e:
+            raise SessionError(e.get_error_code())
 
 
     def closeFile(self, treeId, fileId):
@@ -268,154 +326,240 @@ class SMBConnection():
         :param HANDLE fileId: a valid handle for the file/directory to be closed
 
         :return: None, raises a SessionError exception if error.
-            
+
         """
-        return self._SMBConnection.close(treeId, fileId)
+        try:
+            return self._SMBConnection.close(treeId, fileId)
+        except (smb.SessionError, smb3.SessionError), e:
+            raise SessionError(e.get_error_code())
 
     def deleteFile(self, shareName, pathName):
         """
         removes a file
 
-        :param string shareName: a valid handle for the share where the file is to be opened
+        :param string shareName: a valid name for the share where the file is to be deleted 
         :param string pathName: the path name to remove
 
         :return: None, raises a SessionError exception if error.
-            
+
         """
-        return self._SMBConnection.remove(shareName, pathName)
+        try:
+            return self._SMBConnection.remove(shareName, pathName)
+        except (smb.SessionError, smb3.SessionError), e:
+            raise SessionError(e.get_error_code())
 
     def createDirectory(self, shareName, pathName ):
         """
         creates a directory
 
-        :param string shareName: a valid handle for the share where the file is to be opened
+        :param string shareName: a valid name for the share where the directory is to be created
         :param string pathName: the path name or the directory to create
 
         :return: None, raises a SessionError exception if error.
-            
+
         """
-        return self._SMBConnection.mkdir(shareName, pathName)
+        try:
+            return self._SMBConnection.mkdir(shareName, pathName)
+        except (smb.SessionError, smb3.SessionError), e:
+            raise SessionError(e.get_error_code())
 
     def deleteDirectory(self, shareName, pathName):
         """
         deletes a directory
 
-        :param string shareName: a valid handle for the share where the file is to be opened
+        :param string shareName: a valid name for the share where directory is to be deleted
         :param string pathName: the path name or the directory to delete
 
         :return: None, raises a SessionError exception if error.
-            
+
         """
-        return self._SMBConnection.rmdir(shareName, pathName)
+        try:
+            return self._SMBConnection.rmdir(shareName, pathName)
+        except (smb.SessionError, smb3.SessionError), e:
+            raise SessionError(e.get_error_code())
 
     def waitNamedPipe(self, treeId, pipeName, timeout = 5):
         """
         waits for a named pipe
 
-        :param HANDLE treeId: a valid handle for the share where the file is to be checked
+        :param HANDLE treeId: a valid handle for the share where the pipe is
         :param string pipeName: the pipe name to check
         :param integer timeout: time to wait for an answer
 
         :return: None, raises a SessionError exception if error.
-            
+
         """
-        return self._SMBConnection.waitNamedPipe(treeId, pipeName, timeout = timeout)
+        try:
+            return self._SMBConnection.waitNamedPipe(treeId, pipeName, timeout = timeout)
+        except (smb.SessionError, smb3.SessionError), e:
+            raise SessionError(e.get_error_code())
 
     def transactNamedPipe(self, treeId, fileId, data, waitAnswer = True):
         """
         writes to a named pipe using a transaction command
 
-        :param HANDLE treeId: a valid handle for the share where the file is to be checked
-        :param HANDLE fileId: a valid handle for the file/directory to be closed
+        :param HANDLE treeId: a valid handle for the share where the pipe is
+        :param HANDLE fileId: a valid handle for the pipe
         :param string data: buffer with the data to write
         :param boolean waitAnswer: whether or not to wait for an answer
 
         :return: None, raises a SessionError exception if error.
-            
+
         """
-        return self._SMBConnection.TransactNamedPipe(treeId, fileId, data, waitAnswer = waitAnswer)
+        try:
+            return self._SMBConnection.TransactNamedPipe(treeId, fileId, data, waitAnswer = waitAnswer)
+        except (smb.SessionError, smb3.SessionError), e:
+            raise SessionError(e.get_error_code())
+
 
     def transactNamedPipeRecv(self):
         """
         reads from a named pipe using a transaction command
 
         :return: data read, raises a SessionError exception if error.
-            
+
         """
-        return self._SMBConnection.TransactNamedPipeRecv()
+        try:
+            return self._SMBConnection.TransactNamedPipeRecv()
+        except (smb.SessionError, smb3.SessionError), e:
+            raise SessionError(e.get_error_code())
 
     def writeNamedPipe(self, treeId, fileId, data, waitAnswer = True):
         """
-        writes to a named pipe 
+        writes to a named pipe
 
-        :param HANDLE treeId: a valid handle for the share where the file is to be checked
-        :param HANDLE fileId: a valid handle for the file/directory to be closed
+        :param HANDLE treeId: a valid handle for the share where the pipe is
+        :param HANDLE fileId: a valid handle for the pipe
         :param string data: buffer with the data to write
         :param boolean waitAnswer: whether or not to wait for an answer
 
         :return: None, raises a SessionError exception if error.
-            
+
         """
-        if self.getDialect() == smb.SMB_DIALECT:
-            return self._SMBConnection.write_andx(treeId, fileId, data, wait_answer = waitAnswer, write_pipe_mode = True)
-        else:
-            return self.writeFile(treeId, fileId, data, 0)
+        try:
+            if self.getDialect() == smb.SMB_DIALECT:
+                return self._SMBConnection.write_andx(treeId, fileId, data, wait_answer = waitAnswer, write_pipe_mode = True)
+            else:
+                return self.writeFile(treeId, fileId, data, 0)
+        except (smb.SessionError, smb3.SessionError), e:
+            raise SessionError(e.get_error_code())
+
 
     def readNamedPipe(self,treeId, fileId, bytesToRead = None ):
         """
-        read from a named pipe 
+        read from a named pipe
 
-        :param HANDLE treeId: a valid handle for the share where the file is to be checked
-        :param HANDLE fileId: a valid handle for the file/directory to be closed
+        :param HANDLE treeId: a valid handle for the share where the pipe resides
+        :param HANDLE fileId: a valid handle for the pipe
         :param integer bytestToRead: amount of data to read
         :param boolean waitAnswer: whether or not to wait for an answer
 
         :return: None, raises a SessionError exception if error.
-            
+
         """
 
-        return self.readFile(treeId, fileId, bytesToRead = bytesToRead)  
+        try:
+            return self.readFile(treeId, fileId, bytesToRead = bytesToRead)
+        except (smb.SessionError, smb3.SessionError), e:
+            raise SessionError(e.get_error_code())
 
-    def getFile(self, shareName, pathName, callback):
+
+    def getFile(self, shareName, pathName, callback, shareAccessMode = None):
         """
         downloads a file
 
-        :param string shareName: a valid handle for the share where the file is to be opened
-        :param string pathName: the path name or the directory to delete
-        :param callback callback: 
+        :param string shareName: name for the share where the file is to be retrieved
+        :param string pathName: the path name to retrieve
+        :param callback callback:
 
         :return: None, raises a SessionError exception if error.
-            
-        """
-        return self._SMBConnection.retr_file(shareName, pathName, callback)
 
-    def putFile(self, shareName, pathName, callback):
+        """
+        try:
+	    if shareAccessMode is None:
+		# if share access mode is none, let's the underlying API deals with it
+            	return self._SMBConnection.retr_file(shareName, pathName, callback)
+	    else:
+            	return self._SMBConnection.retr_file(shareName, pathName, callback, shareAccessMode = shareAccessMode)
+        except (smb.SessionError, smb3.SessionError), e:
+            raise SessionError(e.get_error_code())
+
+    def putFile(self, shareName, pathName, callback, shareAccessMode = None):
         """
         uploads a file
 
-        :param string shareName: a valid handle for the share where the file is to be opened
-        :param string pathName: the path name or the directory to delete
-        :param callback callback: 
+        :param string shareName: name for the share where the file is to be uploaded
+        :param string pathName: the path name to upload
+        :param callback callback:
 
         :return: None, raises a SessionError exception if error.
-            
+
         """
-        return self._SMBConnection.stor_file(shareName, pathName, callback)
+        try:
+	    if shareAccessMode is None:
+		# if share access mode is none, let's the underlying API deals with it
+            	return self._SMBConnection.stor_file(shareName, pathName, callback)
+	    else: 
+            	return self._SMBConnection.stor_file(shareName, pathName, callback, shareAccessMode)
+        except (smb.SessionError, smb3.SessionError), e:
+            raise SessionError(e.get_error_code())
 
     def rename(self, shareName, oldPath, newPath):
         """
         rename a file/directory
 
-        :param string shareName: a valid handle for the share where the file is to be opened
+        :param string shareName: name for the share where the files/directories are
         :param string oldPath: the old path name or the directory/file to rename
         :param string newPath: the new path name or the directory/file to rename
 
         :return: True, raises a SessionError exception if error.
-            
+
         """
 
-        return self._SMBConnection.rename(shareName, oldPath, newPath)
+        try:
+            return self._SMBConnection.rename(shareName, oldPath, newPath)
+        except (smb.SessionError, smb3.SessionError), e:
+            raise SessionError(e.get_error_code())
 
     def setTimeout(self, timeout):
-        return self._SMBConnection.set_timeout(timeout)
- 
+        try:
+            return self._SMBConnection.set_timeout(timeout)
+        except (smb.SessionError, smb3.SessionError), e:
+            raise SessionError(e.get_error_code())
+
+    def getSessionKey(self):
+        if self.getDialect() == smb.SMB_DIALECT:
+            return self._SMBConnection.get_session_key()
+        else:
+            return self._SMBConnection.getSessionKey()
+
+    def setSessionKey(self, key):
+        if self.getDialect() == smb.SMB_DIALECT:
+            return self._SMBConnection.set_session_key(key)
+        else:
+            return self._SMBConnection.setSessionKey(key)
+
+class SessionError(Exception):
+    """
+    This is the exception every client should catch regardless of the underlying
+    SMB version used. We'll take care of that. NETBIOS exceptions are NOT included,
+    since all SMB versions share the same NETBIOS instances.
+    """
+    def __init__( self, error = 0, packet=0):
+        Exception.__init__(self)
+        self.error = error
+        self.packet = packet
+       
+    def getErrorCode( self ):
+        return self.error
+
+    def getErrorPacket( self ):
+        return self.packet
+
+    def getErrorString( self ):
+        return nt_errors.ERROR_MESSAGES[self.error]
+
+    def __str__( self ):
+        return 'SMB SessionError: %s(%s)' % (nt_errors.ERROR_MESSAGES[self.error])
+
+
